var canvas = document.getElementById("canvas");
var ctx = canvas.getContext("2d");

var TILE_SIZE = 55;
var PEEP_SIZE = 50;

var GRID_SIZE = 10;

var DIAGONAL_SQUARED = (TILE_SIZE+5)*(TILE_SIZE+5) + (TILE_SIZE+5)*(TILE_SIZE+5);

var assetsLeft = 0;
var onImageLoaded = function(){
	assetsLeft--;
};

var images = {};
function addAsset(name,src){
	assetsLeft++;
	images[name] = new Image();
	images[name].onload = onImageLoaded;
	images[name].src = src;
}
addAsset("yayTriangle","../img/yay_triangle.png");
addAsset("yayTriangleBlink","../img/yay_triangle_blink.png");
addAsset("mehTriangle","../img/meh_triangle.png");
addAsset("sadTriangle","../img/sad_triangle.png");
addAsset("yaySquare","../img/yay_square.png");
addAsset("yaySquareBlink","../img/yay_square_blink.png");
addAsset("mehSquare","../img/meh_square.png");
addAsset("sadSquare","../img/sad_square.png");

var IS_PICKING_UP = false;
var lastMouseX, lastMouseY;

function Draggable(x,y){
	
	var self = this;
	self.x = x;
	self.y = y;
	self.gotoX = x;
	self.gotoY = y;

	var offsetX, offsetY;
	var pickupX, pickupY;
	self.pickup = function(){

		IS_PICKING_UP = true;

		pickupX = (Math.floor(self.x/TILE_SIZE)+0.5)*TILE_SIZE * ZOOM_RATE;
		pickupY = (Math.floor(self.y/TILE_SIZE)+0.5)*TILE_SIZE * ZOOM_RATE;
		offsetX = Mouse.x-self.x * ZOOM_RATE;
		offsetY = Mouse.y-self.y * ZOOM_RATE;
		self.dragged = true;

		// Dangle
		self.dangle = 0;
		self.dangleVel = 0;

		// Draw on top
		var index = draggables.indexOf(self);
		draggables.splice(index,1);
		draggables.push(self);

	};

	self.drop = function(){

		IS_PICKING_UP = false;

		var px = Math.floor((Mouse.x - offsetX)/(TILE_SIZE * ZOOM_RATE));
		var py = Math.floor((Mouse.y - offsetY)/(TILE_SIZE * ZOOM_RATE));
		if(px<0) px=0;
		if(px>=GRID_SIZE) px=GRID_SIZE-1;
		if(py<0) py=0;
		if(py>=GRID_SIZE) py=GRID_SIZE-1;
		var potentialX = (px+0.5)*TILE_SIZE * ZOOM_RATE;
		var potentialY = (py+0.5)*TILE_SIZE * ZOOM_RATE;

		var spotTaken = false;
		for(var i=0;i<draggables.length;i++){
			var d = draggables[i];
			if(d==self) continue;
			var dx = d.x * ZOOM_RATE-potentialX;
			var dy = d.y * ZOOM_RATE-potentialY;
			if(dx*dx+dy*dy<10){
				spotTaken=true;
				break;
			}
		}

		if(spotTaken){
			self.gotoX = pickupX / ZOOM_RATE;
			self.gotoY = pickupY / ZOOM_RATE;
		}else{
			self.gotoX = potentialX / ZOOM_RATE;
			self.gotoY = potentialY / ZOOM_RATE;
			if(pickupX != potentialX || pickupY != potentialY){
				GRID[py][px] = GRID[self.ty][self.tx];
				GRID[self.ty][self.tx] = 0;
				self.ty = py;
				self.tx = px;
				
				if(typeof(stepEvent) == "function"){
					stepEvent();
				}
			}
		}

		self.dragged = false;

	}

	var lastPressed = false;
	self.update = function(){

		// Shakiness?
		self.shaking = false;
		self.bored = false;
		if(!self.dragged){
			var neighbors = 0;
			var same = 0;
			for(var i=0;i<draggables.length;i++){
				var d = draggables[i];
				if(d==self) continue;
				var dx = d.x-self.x;
				var dy = d.y-self.y;
				if(dx*dx+dy*dy<DIAGONAL_SQUARED){
					neighbors++;
					if(d.color==self.color){
						same++;
					}
				}
			}
			if(neighbors>0 && (same/neighbors)<0.33){
				self.shaking = true;
			}
			if(neighbors==0 || (same/neighbors)>0.99){
				//self.bored = true;
			}
		}

		// Dragging
		if(!self.dragged){
			if(self.shaking && Mouse.pressed && !lastPressed){
				var dx = Mouse.x-self.x * ZOOM_RATE;
				var dy = Mouse.y-self.y	* ZOOM_RATE;
				if(Math.abs(dx)<PEEP_SIZE * ZOOM_RATE /2 && Math.abs(dy)<PEEP_SIZE * ZOOM_RATE /2){
					self.pickup();
				}
			}
		}else{
			self.gotoX = (Mouse.x - offsetX) / ZOOM_RATE;
			self.gotoY = (Mouse.y - offsetY) / ZOOM_RATE;
			if(!Mouse.pressed){
				self.drop();
			}
		}
		lastPressed = Mouse.pressed;

		// Going to where you should
		self.x = self.x*0.5 + self.gotoX*0.5;
		self.y = self.y*0.5 + self.gotoY*0.5;

	};

	self.frame = 0;
	self.blinking=0;
	self.draw = function(){
		ctx.save();
		ctx.translate(self.x,self.y);
		if(self.shaking){
			self.frame+=0.07;
			ctx.translate(0,20);
			ctx.rotate(Math.sin(self.frame-(self.x+self.y)/200)*Math.PI*0.05);
			ctx.translate(0,-20);
		}

		// Blinking
		if(Math.random()<0.01){
			self.blinking=10;
		}

		// Draw thing
		var img;
		if(self.color=="triangle"){
			if(self.shaking){
				img = images.sadTriangle;
			}else if(self.bored){
				img = images.mehTriangle;
			}else{
				if(self.blinking>0){
					self.blinking--;
					img = images.yayTriangleBlink;
				}else{
					img = images.yayTriangle;
				}
			}
		}else{
			if(self.shaking){
				img = images.sadSquare;
			}else if(self.bored){
				img = images.mehSquare;
			}else{
				if(self.blinking>0){
					self.blinking--;
					img = images.yaySquareBlink;
				}else{
					img = images.yaySquare;
				}
			}
		}
		
		// Dangle
		if(self.dragged){
			self.dangle += (lastMouseX-Mouse.x)/100;
			ctx.rotate(-self.dangle);
			self.dangleVel += self.dangle*(-0.02);
			self.dangle += self.dangleVel;
			self.dangle *= 0.9;
		}

		ctx.drawImage(img,-PEEP_SIZE/2,-PEEP_SIZE/2,PEEP_SIZE,PEEP_SIZE);
		ctx.restore();
	};

}

var draggables;
function reset(){
	draggables = [];
	for(var x=0;x<GRID.length;x++){
		for(var y=0;y<GRID[x].length;y++){
			if(GRID[x][y] > 0){
				var draggable = new Draggable((y+0.5)*TILE_SIZE, (x+0.5)*TILE_SIZE);
				draggable.ty = x;
				draggable.tx = y;
				draggable.color = (GRID[x][y] == 2) ? "triangle" : "square";
				draggables.push(draggable);
			}
		}
	}
}
reset();

function resetClear(){
	draggables = [];
	for(var x=0;x<DEFGRID.length;x++){
		for(var y=0;y<DEFGRID[x].length;y++){
			if(DEFGRID[x][y] > 0){
				var draggable = new Draggable((y+0.5)*TILE_SIZE, (x+0.5)*TILE_SIZE);
				draggable.ty = x;
				draggable.tx = y;
				draggable.color = (DEFGRID[x][y] == 2) ? "triangle" : "square";
				draggables.push(draggable);
			}
		}
	}
}

function render(){

	if(assetsLeft>0) return;
	
	// Draw
	Mouse.isOverDraggable = IS_PICKING_UP;
	ctx.clearRect(0,0,canvas.width,canvas.height);
	for(var i=0;i<draggables.length;i++){
		var d = draggables[i];
		d.update();

		if(d.shaking){
			var dx = Mouse.x-d.x * ZOOM_RATE;
			var dy = Mouse.y-d.y * ZOOM_RATE;
			if(Math.abs(dx)<PEEP_SIZE * ZOOM_RATE/2 && Math.abs(dy)<PEEP_SIZE * ZOOM_RATE/2){
				Mouse.isOverDraggable = true;
			}
		}

	}
	for(var i=0;i<draggables.length;i++){
		draggables[i].draw();
	}

	// Mouse
	lastMouseX = Mouse.x;
	lastMouseY = Mouse.y;

	// Done?
	if(isDone()){
		doneBuffer--;
		if(doneBuffer==0){
			doneAnimFrame = 30;
			console.log("DONE");
			
			//完成事件
			if(typeof(doneEvent) == "function"){
				doneEvent();
			}
		}
	}else{
		doneBuffer = 30;
	}
	if(doneAnimFrame>0){
		doneAnimFrame--;
		var opacity = ((doneAnimFrame%15)/15)*0.2;
		canvas.style.background = "rgba(255,255,255,"+opacity+")";
	}else{
		canvas.style.background = "none";
	}

}

var doneAnimFrame = 0;
var doneBuffer = 30;
function isDone(){
	if(Mouse.pressed) return false;
	for(var i=0;i<draggables.length;i++){
		var d = draggables[i];
		if(d.shaking) return false;
	}
	return true;
}

////////////////////
// ANIMATION LOOP //
////////////////////
window.requestAnimFrame = window.requestAnimationFrame ||
	window.webkitRequestAnimationFrame ||
	window.mozRequestAnimationFrame ||
	function(callback){ window.setTimeout(callback, 1000/60); };
(function animloop(){
	requestAnimFrame(animloop);
	if(window.IS_IN_SIGHT){
		render();
	}
})();
window.IS_IN_SIGHT = false;

